#!/usr/bin/env python
#
#  For more information, please see: http://software.sci.utah.edu
# 
#  The MIT License
# 
#  Copyright (c) 2009 Scientific Computing and Imaging Institute,
#  University of Utah.
# 
#  
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
# 
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
# 
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
#  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
#  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
#

import os
import sys
from sys import argv
import subprocess
import glob
import time

print_only = False
use_shell = True
if sys.platform == "win32" :
    use_shell = False

combined_prefix = "particle-union"

def assert_exists(file):
    if not(os.path.exists(file)):
        print "No such file %s. Perhaps you need to run a previous stage."%file
        sys.exit(-1)

def do_system(cmmd) :
    if print_only :
        print cmmd
    else :
        ret = -1
        try:
            ret = subprocess.call(cmmd, shell=use_shell)
            if (ret < 0):
                sys.exit(ret)
        except OSError, e:
            print "Execution failed with error ", e
            sys.exit(ret)

def make_nodes_from_dir(d) :
    filename = '%s/%s.pts' % (d, combined_prefix)
    if os.path.exists(filename) :
        os.remove(filename)

    # Basically does the same thing as 'cat'
    pts_file = open('%s/%s.pts' % (d, combined_prefix), 'w')
    for f in glob.glob('%s/particle_params*.pts' % d):
        f = open(f, 'r')
        pts_file.writelines(f.readlines())
        f.close()
    pts_file.close()

    cmmd = r'"%spts2node" %s/%s.pts %s/%s.node' % (binary_path, d, combined_prefix, d, combined_prefix)
    do_system(cmmd)

# must be called after combined is created.
def make_surfs(d) :

    assert_exists("medial_axis_param_file.txt")

    i = 0
    for nm in mat_names :
        cmmd = r'"%sremovetets" %s/%s.1 medial_axis_param_file.txt %s/%s.m %d' % (binary_path, d, combined_prefix, d, nm, i)
        print cmmd
        do_system(cmmd)

        cmmd = r'"%sMtoTriSurfField" %s/%s.m %s/%s.ts.fld' % (binary_path, d, nm, d, nm)
        print cmmd
        do_system(cmmd)
        i = i + 1

def make_combined_surf(d) :

    assert_exists("medial_axis_param_file.txt")

    cmmd = r'"%stetgen" %s/%s.node' % (binary_path, d, combined_prefix)
    print cmmd
    do_system(cmmd)
    
  
    cmmd = r'"%sremovetetsall" %s/%s.1 medial_axis_param_file.txt %s/%s.m 0' % (binary_path, d, combined_prefix, d, combined_prefix)
    print cmmd
    do_system(cmmd)

    cmmd = r'"%sMtoTriSurfField" %s/%s.m %s/%s.ts.fld' % (binary_path, d, combined_prefix, d, combined_prefix)
    print cmmd
    do_system(cmmd)
    
if __name__ == '__main__' :


# Check that we got the right arguments......
    if len(argv) < 2 or not(os.path.exists(argv[1])):
        print 'Usage: %s [model_config] [binary_path]' % argv[0];
        exit(1)

    model_config = argv[1]
    execfile(model_config)
    model_path, dummy = os.path.split(model_config)
    
    if not(os.path.isabs(model_input_file)) :
      model_input_file = os.path.normpath(os.path.join(model_path, model_input_file))

    if not(os.path.isabs(model_output_path)) :
      model_output_path = os.path.normpath(os.path.join(model_path, model_output_path))
   
    if len(argv) > 2:
        binary_path = argv[2]
        if binary_path[len(binary_path) - 1] != os.sep:
            binary_path += os.sep
    else:
        binary_path = ""  

    #sys.path.append('%s/lib' % scirun_bin)

# Done checking arguments

    cur_dir = os.getcwd();
    os.chdir(model_output_path);

    start_time = time.time()

    dir_name = 'junctions'
    make_nodes_from_dir(dir_name) 
    make_combined_surf(dir_name) 
    make_surfs(dir_name) 

    stop_time = time.time()
    f = open("build-material-interface-mesh-runtime.txt","w")
    f.write("%lf" % (stop_time-start_time))
    f.close()

    os.chdir(cur_dir)
