#!/usr/bin/env python
#
#  For more information, please see: http://software.sci.utah.edu
# 
#  The MIT License
# 
#  Copyright (c) 2009 Scientific Computing and Imaging Institute,
#  University of Utah.
# 
#  
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
# 
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
# 
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
#  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
#  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
#

import os
import sys
from sys import argv
import subprocess
import glob
import time

combined_prefix = "particle-union"

print_only = False
use_shell = True
if sys.platform == "win32" :
    use_shell = False

def assert_exists(file):
    if not(os.path.exists(file)):
        print "No such file %s. Perhaps you need to run a previous stage."%file
        sys.exit(-1)

def do_system(cmmd) :
    if print_only :
        print cmmd
    else :
        ret = -1
        try:
            ret = subprocess.call(cmmd, shell=use_shell)
            if (ret < 0):
                sys.exit(ret)
        except OSError, e:
            print "Execution failed with error ", e
            sys.exit(ret)

# must be called after combined is created.
def make_vols(d) :

    i = 0
    tetgen_solo_vol_flags = tetgen_joined_vol_flags
    tet_fname = "%s/tets-all.fld " % d
    tet_trans_fname = "%s/tets-all_transformed.fld " % d
    join_fields_cmmd = r'"%sJoinFields" -indexdata %s' % (binary_path, tet_fname)
    join_transformed_fields_cmmd = r'"%sJoinFields" -indexdata %s' % (binary_path, tet_trans_fname)
    for nm in mat_names :
        tet_fname = "%s/%s.tets.fld" % (d, nm)
        mat_surf = "%s/%s.ts.fld" % (d,nm)
        assert_exists(mat_surf)
        cmmd = r'"%sInterfaceWithTetGen" -cmmd_line %s -main %s -out %s' % (binary_path, tetgen_solo_vol_flags, mat_surf, tet_fname)
        print(cmmd)
        do_system(cmmd)
        transform_with_transform(tet_fname)  
        join_fields_cmmd += "%s/%s.tets.fld " % (d,nm)
        join_transformed_fields_cmmd += "%s/%s.tets_transformed.fld " % (d, nm)
        i = i + 1
    do_system(join_fields_cmmd)
    do_system(join_transformed_fields_cmmd)



def make_combined_vol(d) :
    combined_surf = "%s/%s.ts.fld" % (d,combined_prefix)
    assert_exists(combined_surf)
    tet_fname = "%s/%s.tets.fld" % (d, combined_prefix)
    cmmd = r'"%sInterfaceWithTetGen" -cmmd_line %s -main %s -out %s' % (binary_path, tetgen_joined_vol_flags, combined_surf, tet_fname)
    do_system(cmmd)


def transform_with_transform(tet_file) :
    assert_exists(tet_file)
    transformed_fname = "%s_transformed.fld" % tet_file[:-4]
    transf = "%s_pad_transform.tf" % model_input_file[:-5]
    dummy,filename = os.path.split(transf)
    transf = os.path.join(model_output_path,filename)
    print "Transforming %s" % tet_file
    trans_cmmd = r'"%sTransformFieldWithTransform" -input %s -transform %s -output %s' % (binary_path, tet_file, transf, transformed_fname)
    do_system(trans_cmmd)
    print "Done Transforming"
    


if __name__ == '__main__' :

# Check that we got the right arguments......
    if len(argv) < 2 or not(os.path.exists(argv[1])):
        print 'Usage: %s [model_config] [binary_path]' % argv[0];
        exit(1)

    model_config = argv[1]
    execfile(model_config)
    
    model_path, dummy = os.path.split(model_config)
    
    if not(os.path.isabs(model_input_file)) :
      model_input_file = os.path.normpath(os.path.join(model_path, model_input_file))

    if not(os.path.isabs(model_output_path)) :
      model_output_path = os.path.normpath(os.path.join(model_path, model_output_path))
   
    
    if len(argv) > 2:
        binary_path = argv[2]
        if binary_path[len(binary_path) - 1] != os.sep:
            binary_path += os.sep
    else:
        binary_path = ""  

# Done checking arguments

    cur_dir = os.getcwd();
    os.chdir(model_output_path);

    start_time = time.time()

    dir_name = 'junctions'
    assert_exists(dir_name)

    make_combined_vol(dir_name) 
    make_vols(dir_name) 

    stop_time = time.time()
    f = open("build-volumetric-mesh-runtime.txt","w")
    f.write("%lf" % (stop_time-start_time))
    f.close()

    os.chdir(cur_dir)
