#
#  For more information, please see: http://software.sci.utah.edu
# 
#  The MIT License
# 
#  Copyright (c) 2009 Scientific Computing and Imaging Institute,
#  University of Utah.
# 
#  
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
# 
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software. 
# 
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
#  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
#  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
#

#
# SEARCH_FOR_SYSTEM_3P( SCIRun_Version
#                       Compiler
#                       Compiler_Version
#                       Bits )
#
# Search for the SCIRun Thirdparty on the SCI file system.
#

MACRO(SEARCH_FOR_SYSTEM_3P version compiler compiler_version bits)

    MESSAGE( "search_for_system_3p: ${CMAKE_SYSTEM_NAME}" )

    IF(NOT WIN32) # There is not a system wide Thirdparty build for Windows.

        # Determine system
        IF( CMAKE_SYSTEM_NAME STREQUAL "IRIX" )
            SET(SCI_ARCH "IRIX64")
        ELSEIF( CMAKE_SYSTEM_NAME STREQUAL "Darwin" )
            EXECUTE_PROCESS( COMMAND uname -p OUTPUT_VARIABLE processor )
            # Uname returns a \n at the end, so we have to cut it off... :(
            STRING( LENGTH ${processor} proc_str_len ) 
            MATH( EXPR proc_str_len "${proc_str_len} - 1" )
            STRING( SUBSTRING ${processor} 0 ${proc_str_len} processor ) 
            SET(SCI_ARCH "Darwin-${processor}")
        ELSE( CMAKE_SYSTEM_NAME STREQUAL "IRIX" )
            SET(SCI_ARCH ${CMAKE_SYSTEM_NAME})
        ENDIF( CMAKE_SYSTEM_NAME STREQUAL "IRIX") 

        # First check if a thirdparty path exists on the sci system
        # based on the version, system architecture, compiler version, 
        # and number of bits
        SET( SCI_3P_DIR_ATTEMPT "/usr/sci/projects/SCIRun/Thirdparty/${version}/${SCI_ARCH}/${compiler}-${compiler_version}-${bits}bit")

        MESSAGE( "SCI_3P_DIR_ATTEMPT is ${SCI_3P_DIR_ATTEMPT}" )

        IF( EXISTS ${SCI_3P_DIR_ATTEMPT}/SCIRUN_THIRDPARTY_VERSION )

            SET( SCIRUN_THIRDPARTY_DIR ${SCI_3P_DIR_ATTEMPT} CACHE PATH "Path to SCIRun's Thirdparty" FORCE )

        ELSE( EXISTS ${SCI_3P_DIR_ATTEMPT}/SCIRUN_THIRDPARTY_VERSION )

            SET( SCIRUN_THIRDPARTY_DIR NOTFOUND CACHE PATH "Path to SCIRun's Thirdparty" FORCE )

        ENDIF( EXISTS ${SCI_3P_DIR_ATTEMPT}/SCIRUN_THIRDPARTY_VERSION )

    ENDIF(NOT WIN32)

ENDMACRO(SEARCH_FOR_SYSTEM_3P)
